"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@babel/core");
function extractTypeProperties(node, types) {
    var properties = [];
    var mapToPropertySignature = function (data) {
        data.forEach(function (prop) {
            if (core_1.types.isTSPropertySignature(prop)) {
                properties.push(prop);
            }
        });
    };
    // Props
    if (core_1.types.isIdentifier(node)) {
        if (types[node.name]) {
            properties.push.apply(properties, types[node.name]);
        }
        // Props
    }
    else if (core_1.types.isTSTypeReference(node)) {
        properties.push.apply(properties, extractTypeProperties(node.typeName, types));
        // interface {}
    }
    else if (core_1.types.isTSInterfaceDeclaration(node)) {
        (node.extends || []).forEach(function (ext) {
            properties.push.apply(properties, extractTypeProperties(ext.expression, types));
        });
        mapToPropertySignature(node.body.body);
        // type = {}
    }
    else if (core_1.types.isTSTypeAliasDeclaration(node)) {
        properties.push.apply(properties, extractTypeProperties(node.typeAnnotation, types));
        // {}
    }
    else if (core_1.types.isTSTypeLiteral(node)) {
        mapToPropertySignature(node.members);
        // Props & {}, Props | {}
    }
    else if (core_1.types.isTSIntersectionType(node) || core_1.types.isTSUnionType(node)) {
        node.types.forEach(function (intType) {
            properties.push.apply(properties, extractTypeProperties(intType, types));
        });
    }
    return properties;
}
exports.default = extractTypeProperties;
