"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var helper_module_imports_1 = require("@babel/helper-module-imports");
var plugin_syntax_typescript_1 = __importDefault(require("@babel/plugin-syntax-typescript"));
var core_1 = require("@babel/core");
// import ts from 'typescript';
var addToClass_1 = __importDefault(require("./addToClass"));
var addToFunctionOrVar_1 = __importDefault(require("./addToFunctionOrVar"));
var extractTypeProperties_1 = __importDefault(require("./extractTypeProperties"));
var BABEL_VERSION = 7;
function isNotTS(name) {
    return name.endsWith('.js') || name.endsWith('.jsx');
}
exports.default = helper_plugin_utils_1.declare(function (api) {
    api.assertVersion(BABEL_VERSION);
    return {
        inherits: plugin_syntax_typescript_1.default,
        manipulateOptions: function (opts, parserOptions) {
            // Inheriting the syntax doesn't seem to define these
            parserOptions.plugins.push('jsx');
        },
        // pre(state: any) {
        //   program = ts.createProgram([state.opts.filename], {});
        //   checker = program.getTypeChecker();
        //   console.log(checker.getSymbolAtLocation(program.getSourceFile(state.opts.filename)!));
        // },
        visitor: {
            Program: {
                enter: function (programPath, state) {
                    if (isNotTS(state.filename)) {
                        return;
                    }
                    // Setup our initial state
                    state.reactImportedName = '';
                    state.propTypesImportedName = '';
                    state.hasPropTypesImport = false;
                    state.componentCount = 0;
                    state.componentTypes = {};
                    // Find existing `react` and `prop-types` imports
                    programPath.node.body.forEach(function (node) {
                        if (!core_1.types.isImportDeclaration(node)) {
                            return;
                        }
                        if (node.source.value === 'prop-types') {
                            state.hasPropTypesImport = true;
                            node.specifiers.forEach(function (spec) {
                                if (core_1.types.isImportDefaultSpecifier(spec) || core_1.types.isImportNamespaceSpecifier(spec)) {
                                    state.propTypesImportedName = spec.local.name;
                                }
                            });
                        }
                        if (node.source.value === 'react') {
                            node.specifiers.forEach(function (spec) {
                                if (core_1.types.isImportDefaultSpecifier(spec) || core_1.types.isImportNamespaceSpecifier(spec)) {
                                    state.reactImportedName = spec.local.name;
                                }
                            });
                        }
                    });
                    // Add `prop-types` import if it does not exist.
                    // We need to do this without a visitor as we need to modify
                    // the AST before anything else has can run.
                    if (!state.hasPropTypesImport && state.reactImportedName) {
                        state.hasPropTypesImport = true;
                        state.propTypesImportedName = helper_module_imports_1.addDefault(programPath, 'prop-types', {
                            nameHint: 'pt',
                        }).name;
                    }
                    programPath.traverse({
                        // `class Foo extends React.Component<Props> {}`
                        // @ts-ignore
                        'ClassDeclaration|ClassExpression': function (path) {
                            var node = path.node;
                            // prettier-ignore
                            var valid = node.superTypeParameters && (
                            // React.Component, React.PureComponent
                            (core_1.types.isMemberExpression(node.superClass) &&
                                core_1.types.isIdentifier(node.superClass.object, { name: state.reactImportedName }) && (core_1.types.isIdentifier(node.superClass.property, { name: 'Component' }) ||
                                core_1.types.isIdentifier(node.superClass.property, { name: 'PureComponent' }))) ||
                                // Component, PureComponent
                                (state.reactImportedName && (core_1.types.isIdentifier(node.superClass, { name: 'Component' }) ||
                                    core_1.types.isIdentifier(node.superClass, { name: 'PureComponent' }))));
                            if (valid) {
                                addToClass_1.default(node, state);
                                state.componentCount += 1;
                            }
                        },
                        // `function Foo(props: Props) {}`
                        FunctionDeclaration: function (path) {
                            var node = path.node;
                            var param = node.params[0];
                            // prettier-ignore
                            var valid = param &&
                                state.reactImportedName &&
                                node.id.name.match(/^[A-Z]/u) && (
                            // (props: Props)
                            (core_1.types.isIdentifier(param) && param.typeAnnotation) ||
                                // ({ ...props }: Props)
                                (core_1.types.isObjectPattern(param) && param.typeAnnotation));
                            if (valid) {
                                addToFunctionOrVar_1.default(path, node.id.name, state);
                                state.componentCount += 1;
                            }
                        },
                        // `interface FooProps {}`
                        // @ts-ignore
                        TSInterfaceDeclaration: function (_a) {
                            var node = _a.node;
                            state.componentTypes[node.id.name] = extractTypeProperties_1.default(node, state.componentTypes);
                        },
                        // `type FooProps = {}`
                        TSTypeAliasDeclaration: function (_a) {
                            var node = _a.node;
                            state.componentTypes[node.id.name] = extractTypeProperties_1.default(node, state.componentTypes);
                        },
                        // `const Foo: React.SFC<Props> = () => {};`
                        VariableDeclaration: function (path) {
                            var node = path.node;
                            if (node.declarations.length === 0) {
                                return;
                            }
                            var id = node.declarations[0].id;
                            if (!id.typeAnnotation || !id.typeAnnotation.typeAnnotation) {
                                return;
                            }
                            var type = id.typeAnnotation.typeAnnotation;
                            // prettier-ignore
                            var valid = core_1.types.isTSTypeReference(type) &&
                                type.typeParameters &&
                                type.typeParameters.params.length > 0 && (
                            // React.SFC, React.StatelessComponent
                            (core_1.types.isTSQualifiedName(type.typeName) &&
                                core_1.types.isIdentifier(type.typeName.left, { name: state.reactImportedName }) &&
                                (core_1.types.isIdentifier(type.typeName.right, { name: 'SFC' }) ||
                                    core_1.types.isIdentifier(type.typeName.right, { name: 'StatelessComponent' }))) ||
                                // SFC, StatelessComponent
                                (state.reactImportedName && (core_1.types.isIdentifier(type.typeName, { name: 'SFC' }) ||
                                    core_1.types.isIdentifier(type.typeName, { name: 'StatelessComponent' }))));
                            if (valid) {
                                addToFunctionOrVar_1.default(path, id.name, state);
                                state.componentCount += 1;
                            }
                        },
                    });
                },
                exit: function (path, state) {
                    if (isNotTS(state.filename) || state.componentCount !== 0) {
                        return;
                    }
                    // Remove the `prop-types` import of no components exist
                    path.get('body').forEach(function (bodyPath) {
                        if (core_1.types.isImportDeclaration(bodyPath.node) &&
                            bodyPath.node.source.value === 'prop-types') {
                            bodyPath.remove();
                        }
                    });
                },
            },
        },
    };
});
